unit HidLogicu;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, Math;

type
  TCells = array[1..9] of array[1..9] of integer;
  TCellSet = RECORD
    CellCount: integer;
    csi, csj: array[1..81] of integer;
    IsIncluded: array[1..9] of array[1..9] of boolean;
  end;
  TNumberPlacement = array[1..81] of TCellSet;
  TNumberSet = RECORD
    NumberCount: integer;
    nsn: array[1..81] of integer;
  end;
  TCellPossibility = array[1..9] of array[1..9] of TNumberSet;
type
  THidatoGrid = class
    private
    public
      Cells : TCells;
      MaxClWntd : integer;
      procedure PlaceIntoStringGrid(var StrGr : TStringGrid);
      procedure GetFromStringGrid(var StrGr : TStringGrid);
      procedure PlaceIntoStringList(var StrL : TStringList);
      procedure GetFromStringList(var StrL : TStringList);
      procedure EmptyCells;
      procedure AddAllForced;
      function Valid: boolean;
      procedure CreateCompleteGrid;
      procedure CreatePuzzle;
    protected
      Proceeded: boolean;
      Unsolvable: boolean;
      RouteDistance: array[1..9] of array[1..9] of array[1..9] of array[1..9] of integer;
               {RouteDistance[i1,j1,i2,j2] geeft de afstand terug tussen (i1,j1) en (i2,j2)
                volgens een zo kort mogelijke route over lege cellen}
      NumberPlacement: TNumberPlacement;
                {NumberPlacement[n].CellCount geeft het aantal cellen terug waar n kan worden geplaatst.
                 NumberPlacement[n].csi[p] en NumberPlacement[n].csj[p] geven de coordinaten terug van de p-de cel
                 uit deze set cellen.
                 NumberPlacement[n].IsIncluded[i,j] geeft terug of n wel of niet binnen cel (i,j)
                 kan worden geplaatst.}
      CellPossibility: TCellPossibility;
                {CellPossibility[i,j].NumberCount geeft het aantal cijfers terug dat in cel (i,j) kan worden geplaatst
                 CellPossibility[i,j].nsn[p] geeft het p-de cijfer terug van deze cijfers.}
      StatCount: integer;
                {StatCount dient - net als NumberPlacement en CellPossibility - binnen de cyclus van AddAllForced
                synchroon te worden bijgewerkt}
      UsedForStraightDistanceExclusion: array[1..81] of boolean;
      function DynCount: integer;
      procedure ExcludeByOneFreeNeighbour;
      procedure ExcludeByRouteDistance;
      procedure ExcludeByStraightDistance;
      procedure ExcludeBySingles;
      procedure InitAddAllForced;
      procedure AddSomeForced;
      procedure ReCalculateNumberPlacementCellPossibility;
      procedure ComputeRouteDistances;
      procedure PlaceNumber(n, i, j: integer);
      procedure InitNumberPlacement;
      procedure InitCellPossibility;
      procedure RemoveFromNumberPlacement(n, i, j: integer);
      procedure RemoveFromCellPossibility(n, i, j: integer);
      procedure ExcludeNumberFromCell(n, i, j: integer);
      procedure AddSomeClues;
      function EqualCellSet(CellSet1, CellSet2: TCellSet): boolean;
      function EqualNumberSet(NumberSet1, NumberSet2: TNumberSet): boolean;
      procedure DebugNumberPlacement(n: integer);
      procedure DebugCellPossibility(i, j: integer);
  end;
type
  THidatoGame = class
    private
    public
      MainGrid: THidatoGrid;
      constructor Create;
      destructor Destroy;override;
    protected
  end;

implementation

var Neighbouri, Neighbourj: array[1..9] of array[1..9] of array[1..8] of integer;
var NeighbourCount: array[1..9] of array[1..9] of integer;
var EmptyCellSet: TCellSet;
var EmptyNumberSet: TNumberSet;
var TooFar: integer;
var StraightDistance: array[1..9] of array[1..9] of array[1..9] of array[1..9] of integer;
var CellSingleton: array[1..9] of array[1..9] of TCellSet;
var NumberSingleton: array[1..81] of TNumberSet;
var AllCellSet: TCellSet;
var AllNumberSet: TNumberSet;

procedure FillCellSingletons;
 var i, j: integer;
begin
  for i := 1 to 9 do
    for j := 1 to 9 do
    begin
      CellSingleton[i,j] := EmptyCellSet;
      CellSingleton[i,j].CellCount := 1;
      CellSingleton[i,j].csi[1] := i;
      CellSingleton[i,j].csj[1] := j;
      CellSingleton[i,j].IsIncluded[i,j] := true;
    end;{-for}
end;

procedure FillNumberSingletons;
  var n: integer;
begin
  for n := 1 to 81 do
  begin
    NumberSingleton[n] := EmptyNumberSet;
    NumberSingleton[n].NumberCount := 1;
    NumberSingleton[n].nsn[1] := n;
  end;{-for}
end;

procedure FillNeighbours;
  var i, j, ni, nj, c: integer;
begin
  for i := 1 to 9 do
    for j := 1 to 9 do
    begin
      NeighbourCount[i,j] := 0;
      for ni := i - 1 to i + 1 do
        for nj := j - 1 to j + 1 do
          if (not ((i = ni) and (j = nj))) and
             ( ni > 0 ) and  ( ni < 10 ) and
             ( nj > 0 ) and  ( nj < 10 ) then
          begin
            NeighbourCount[i,j] := NeighbourCount[i,j] + 1;
            c := NeighbourCount[i,j];
            Neighbouri[i,j,c] := ni;
            Neighbourj[i,j,c] := nj;
          end;{-if}
    end;{-for}
end;

procedure FillEmptyNumberSet;
  var n: integer;
begin
  EmptyNumberSet.NumberCount := 0;
  for n := 1 to 81 do EmptyNumberSet.nsn[n] := 0;
end;

procedure FillEmptyCellSet;
  var i, j, n: integer;
begin
  EmptyCellSet.CellCount := 0;
  for i := 1 to 9 do
    for j := 1 to 9 do
      EmptyCellSet.IsIncluded[i,j] := false;
  for n := 1 to 81 do
  begin
    EmptyCellSet.csi[n] := 0;
    EmptyCellSet.csj[n] := 0;
  end;
end;

procedure FillTooFar;
begin
  TooFar := 100;
end;

procedure FillStraightDistances;
  var i1, i2, j1, j2: integer;
begin
  for i1 := 1 to 9 do
    for i2 := 1 to 9 do
      for j1 := 1 to 9 do
        for j2 := 1 to 9 do
        begin
          StraightDistance[i1,j1,i2,j2] := max(abs(i1 - i2),abs(j1 - j2));
        end;{-for}
//  Showmessage(IntToStr(StraightDistance[1,2,4,8]));
end;

procedure FillAllCellSet;
  var i, j, p: integer;
begin
  AllCellSet.CellCount := 81;
  p := 0;
  for i := 1 to 9 do
    for j := 1 to 9 do
    begin
      p := p + 1;
      AllCellSet.IsIncluded[i,j] := true;
      AllCellSet.csi[p] := i;
      AllCellSet.csj[p] := j;
    end;
end;

procedure FillAllNumberSet;
  var p: integer;
begin
  AllNumberSet.NumberCount := 81;
  for p := 1 to 81 do AllNumberSet.nsn[p] := p;
end;

procedure FillConstants;
begin
  FillNeighbours;
  FillEmptyCellSet;
  FillEmptyNumberSet;
  FillTooFar;
  FillStraightDistances;
  FillCellSingletons;{na FillEmptyCellSet!!}
  FillNumberSingletons;{na FillEmptyNumberSet!!}
  FillAllCellSet;
  FillAllNumberSet;
end;

procedure THidatoGrid.EmptyCells;
  var i, j : integer;
begin
  for i := 1 to 9 do
    for j := 1 to 9 do
      self.Cells[i,j] := 0;
end;

function THidatoGrid.DynCount: integer;
  var i, j: integer;
begin
  result := 0;
  for i := 1 to 9 do
    for j := 1 to 9 do
      if self.Cells[i,j] > 0 then result := result + 1;
end;

procedure THidatoGrid.ExcludeByOneFreeNeighbour;
  var i, j: integer;
  var e, i2, j2, n: integer;
  var NeighbourNumbers: TNumberSet;
  var FreeNeighbourCount: integer;
  var NextToNeighbourNumber: boolean;
begin
{Als een lege cel (i,j) precies een lege buurcel heeft, dan kunnen in cel(i,j) - naast 1 of 81 -
 alleen nog cijfers geplaats worden, die predecessor of successor zijn van een der
 in de buurcellen ingevulde cijfers.}
  for i := 1 to 9 do
    for j := 1 to 9 do
      if self.Cells[i,j] = 0 then
      begin
      { --- begin verwerking lege cel (i,j) --- }
        FreeNeighbourCount := 0;
        NeighbourNumbers := EmptyNumberSet;
        {eerst alle buurcellen aflopen:}
        for e := 1 to NeighbourCount[i,j] do
        begin
          i2 := Neighbouri[i,j,e];
          j2 := Neighbourj[i,j,e];
          if self.Cells[i2,j2] = 0 then
          begin
            FreeNeighbourCount := FreeNeighBourCount + 1;
            //als FreeNeighbourCount > 1 dan mag je eigenlijk door naar volgende lege cel
          end else
          begin
            NeighbourNumbers.NumberCount := NeighbourNumbers.NumberCount + 1;
            NeighbourNumbers.nsn[NeighbourNumbers.NumberCount] := self.Cells[i2,j2];
          end;{if-then-else}
        end;
        {als er dan precies een lege buurcel is: alle cijfers n aflopen m.u.v. 1 en 81}
        if FreeNeighbourCount = 1 then
          for n := 2 to 80 do
          begin
            NextToNeighbourNumber := false;
            for e := 1 to NeighbourNumbers.NumberCount do
              if abs(NeighbourNumbers.nsn[e] - n) = 1 then NextToNeighbourNumber := true;
            if not NextToNeighbourNumber then
            begin
//              ShowMessage('ByOneNeighBour: ' + IntToStr(n) + ' ' + IntToStr(i) + ' '+ IntToStr(j) + ' ');
              self.ExcludeNumberFromCell(n,i,j);
            end;
          end;
      { --- einde verwerking lege cel (i,j) --- }
      end;
end;

procedure THidatoGrid.ExcludeByRouteDistance;
  var FreeIntervals: array[1..81] of TNumberSet;
  var FreeIntervalCount: integer;
  var li, lj, ui, uj, n, idx, i2, j2: integer;
  var PreviousPlaced, CurrentPlaced: boolean;
  var UpperLimit, LowerLimit: integer;
begin
  for idx := 1 to 81 do FreeIntervals[idx] := EmptyNumberSet;
  FreeIntervalCount := 0;
  self.ComputeRouteDistances;

  PreviousPlaced := true;

  for n := 1 to 81 do
  begin
    CurrentPlaced :=
       ( self.Cells[NumberPlacement[n].csi[1],NumberPlacement[n].csj[1]] = n );
    if not CurrentPlaced then
    begin
      if PreviousPlaced then
      begin
        FreeIntervalCount := FreeIntervalCount + 1;
        FreeIntervals[FreeIntervalCount].NumberCount := 0;
      end;
      with FreeIntervals[FreeIntervalCount] do
      begin
        NumberCount := NumberCount + 1;
        nsn[NumberCount] := n;
      end;{-with}
    end;
    PreviousPlaced := CurrentPlaced;
  end;

  for idx := 1 to FreeIntervalCount do
  begin
    LowerLimit := FreeIntervals[idx].nsn[1] - 1;
    if LowerLimit > 0 then
    begin
      li := self.NumberPlacement[LowerLimit].csi[1];
      lj := self.NumberPlacement[LowerLimit].csj[1];
    end;
    UpperLimit := FreeIntervals[idx].nsn[FreeIntervals[idx].NumberCount] + 1;
    if UpperLimit < 82 then
    begin
      ui := self.NumberPlacement[UpperLimit].csi[1];
      uj := self.NumberPlacement[UpperLimit].csj[1];
    end;
    for n := LowerLimit + 1 to UpperLimit - 1 do
      for i2 := 1 to 9 do
        for j2 := 1 to 9 do
        begin
          if LowerLimit > 0 then
          begin
            if n - LowerLimit < RouteDistance[li,lj,i2,j2] then
            begin
              self.ExcludeNumberFromCell(n,i2,j2);
            end;
          end;
          if UpperLimit < 82 then
          begin
            if UpperLimit - n < RouteDistance[ui,uj,i2,j2] then
            begin
              self.ExcludeNumberFromCell(n,i2,j2);
            end;
          end;
        end;
  end;
end;

procedure THidatoGrid.ExcludeByStraightDistance;
  var i, i2, j, j2, n, n2, d: integer;
begin
  for i := 1 to 9 do
    for j := 1 to 9 do
      if self.Cells[i,j] > 0 then
      begin
        n := self.Cells[i,j];
        if not self.UsedForStraightDistanceExclusion[n] then
        begin
          for i2 := 1 to 9 do
            for j2 := 1 to 9 do
            begin
              d := StraightDistance[i,j,i2,j2];
              for n2 := 1 to 81 do
                if abs(n - n2) < d then
                begin
                  self.ExcludeNumberFromCell(n2,i2,j2);
                end;
            end;
          self.UsedForStraightDistanceExclusion[n] := true;
        end;{-if}
      end;{-if}
end;

procedure THidatoGrid.ExcludeBySingles;
  var i, j, n: integer;
begin
  for i := 1 to 9 do
    for j := 1 to 9 do
      if self.CellPossibility[i,j].NumberCount = 1 then
      begin
        n := self.CellPossibility[i,j].nsn[1];
        if self.Cells[i,j] = 0 then self.PlaceNumber(n,i,j);
      end;
  for n := 1 to 81 do
    if self.NumberPlacement[n].CellCount = 1 then
    begin
      i := self.NumberPlacement[n].csi[1];
      j := self.NumberPlacement[n].csj[1];
      if self.Cells[i,j] = 0 then self.PlaceNumber(n,i,j);
    end;
end;

procedure THidatoGrid.AddSomeForced;
begin
  self.ExcludeByOneFreeNeighbour;
  self.ExcludeByStraightDistance;
  self.ExcludeByRouteDistance;
  self.ExcludeBySingles;
end;

procedure THidatoGrid.ReCalculateNumberPlacementCellPossibility;
  var i, j, n: integer;
begin
{NumberPlacement en CellPossibility worden afgeleid uit Cells
 hierbij wordt gebruikgemaakt van PlaceNumber}
  self.InitNumberPlacement;
  self.InitCellPossibility;
  for i := 1 to 9 do
    for j := 1 to 9 do
      if self.Cells[i,j] > 0 then
      begin
        n := self.Cells[i,j];
        self.PlaceNumber(n,i,j);
      end;
end;

procedure THidatoGrid.InitAddAllForced;
  var n: integer;
begin
 {Alle voor AddAllForced benodigde initialisaties}
  self.ReCalculateNumberPlacementCellPossibility;
  self.StatCount := self.DynCount;
  for n := 1 to 81 do UsedForStraightDistanceExclusion[n] := false;
end;

procedure THidatoGrid.AddAllForced;
begin
  self.InitAddAllForced;
  self.Unsolvable := false;
  repeat
    self.Proceeded := false;
    self.AddSomeForced;
  until not (self.Proceeded and not self.Unsolvable);
end;

function THidatoGrid.Valid: boolean;
  var i, j, i2, j2, n, n2: integer;
begin
  result := true;
  for i := 1 to 9 do
    for j := 1 to 9 do
      if self.Cells[i,j] > 0 then
      begin
        n := self.Cells[i,j];
        for i2 := 1 to 9 do
          for j2 := 1 to 9 do
            if (i2 <> i) or (j2 <> j) then
              if self.Cells[i2,j2] > 0 then
              begin
                n2 := self.Cells[i2,j2];
                if StraightDistance[i,j,i2,j2] > abs(n2 - n) then result := false;
              end;
      end;
end;

procedure THidatoGrid.AddSomeClues;
  var e, p, i, j, n: integer;
begin
  Application.ProcessMessages;
  p := RandomRange(1,82);
  i := ((p - 1) mod 9) + 1;
  j := ((p - 1) div 9) + 1;
  while Cells[i,j] > 0 do
  begin
    p := RandomRange(1,82);
    i := ((p - 1) mod 9) + 1;
    j := ((p - 1) div 9) + 1;
  end;

  e := RandomRange(1,CellPossibility[i,j].NumberCount + 1);

  n := CellPossibility[i,j].nsn[e];
  PlaceNumber(n,i,j);

  AddAllForced;
end;

procedure THidatoGrid.CreateCompleteGrid;
  var CheckCells : TCells;
  var CheckCount : integer;
begin
  if Valid then
  begin
    randomize;
    CheckCells := self.Cells;
    CheckCount := self.DynCount;
    AddAllForced;
    if not UnSolvable then
      while StatCount < 81 do
      begin
        while not (UnSolvable or ( StatCount = 81 )) do AddSomeClues;
        if UnSolvable then
        begin
          Application.ProcessMessages;
          Cells := CheckCells;
          StatCount := CheckCount;
          AddAllForced;
          randomize;
        end;{-if}
      end;{-while}
  end;
end;

procedure THidatoGrid.CreatePuzzle;
  var RndPm, InitPm: array[1..81] of integer;
  var RndPmFilled, InitPmFilled: array[1..81] of boolean;
  var CompleteCells, npCells, cpCells: TCells;
  var npCount, cpCount, p, r, i, j: integer;

  procedure CreatePermutation;{sub within CreatePuzzle}
    var NumberOfFilled, i: integer;
  begin
    randomize;
    RndPm := InitPm;
    RndPmFilled := InitPmFilled;
    NumberOfFilled := 0;
    while NumberOfFilled < 81 do
    begin
      i := RandomRange(1,82);
      while RndPmFilled[i] do i := RandomRange(1,82);
      NumberOfFilled := NumberOfFilled + 1;
      RndPm[i] := NumberOfFilled;
      RndPmFilled[i] := true;
    end;{-while}
  end;

  procedure FillInitPm;{sub within CreatePuzzle}
    var i: integer;
  begin
    for i := 1 to 81 do
    begin
      InitPm[i] := 0;
      InitPmFilled[i] := false
    end;{-for}
  end;

  procedure ResetSomeClues;{sub within CreatePuzzle}
    var p, i, j, AddedClues: integer;
  begin
    AddedClues := 0;
    while ( StatCount < 81 ) and ( AddedClues < 5 ) do
    begin
      p := RandomRange(1,82);
      i := AllCellSet.csi[p];
      j := AllCellSet.csj[p];
      while Cells[i,j] > 1 do
      begin
        p := RandomRange(1,82);
        i := AllCellSet.csi[p];
        j := AllCellSet.csj[p];
      end;
      Cells[i,j] := CompleteCells[i,j];
      StatCount := StatCount + 1;
      AddedClues := AddedClues + 1;
    end
  end;

begin
  CreateCompleteGrid;
  CompleteCells := Cells;
  FillInitPm;

  while StatCount > MaxClWntd do
  begin
    Application.ProcessMessages;
    ResetSomeClues;
    CreatePermutation;
    cpCells := Cells;
    cpCount := StatCount;

    for r := 1 to 81 do
    begin
      p := RndPm[r];
      i := AllCellSet.csi[p];
      j := AllCellSet.csj[p];

      if Cells[i,j] > 0 then
      begin
        Cells[i,j] := 0;
        StatCount := StatCount - 1;
        npCells := Cells;
        npCount := StatCount;
        AddAllForced;
        if StatCount = 81 then
        begin
          cpCells := npCells;
          cpCount := npCount;
        end;{-if}
        Cells := cpCells;
        StatCount := cpCount;
      end;{-if}
    end;{-for}
  end;{-while}
end;

procedure THidatoGrid.ComputeRouteDistances;
  var BinnenRing, BuitenRing: TCellSet;
  var i1, j1, i2, j2: integer;
  var r, n, hRouteDistance: integer;
begin
  for i1 := 1 to 9 do
    for j1 := 1 to 9 do
    begin
      for i2 := 1 to 9 do
        for j2 := 1 to 9 do
          if ((i1 = i2) and (j1 = j2)) then RouteDistance[i1,j1,i2,j2] := 0
                                       else RouteDistance[i1,j1,i2,j2] := TooFar;

      BinnenRing := EmptyCellSet;
      BinnenRing.CellCount := 1;
      BinnenRing.csi[1] := i1;
      BinnenRing.csj[1] := j1;
      BinnenRing.IsIncluded[i1,j1] := true;
      hRouteDistance := 1;
      repeat
        BuitenRing := EmptyCellSet;
        for r := 1 to BinnenRing.CellCount do
          for n := 1 to NeighbourCount[BinnenRing.csi[r],BinnenRing.csj[r]] do
          begin
            i2 := Neighbouri[BinnenRing.csi[r],BinnenRing.csj[r],n];
            j2 := Neighbourj[BinnenRing.csi[r],BinnenRing.csj[r],n];
            if RouteDistance[i1,j1,i2,j2] = TooFar then
            begin
              RouteDistance[i1,j1,i2,j2] := hRouteDistance;
              if (self.Cells[i2,j2] = 0) and not BuitenRing.IsIncluded[i2,j2] then
              begin
                BuitenRing.CellCount := BuitenRing.CellCount + 1;
                BuitenRing.csi[BuitenRing.CellCount] := i2;
                BuitenRing.csj[BuitenRing.CellCount] := j2;
                BuitenRing.IsIncluded[i2,j2] := true;
              end;{-if}
            end;{-if}
          end;{-for}
        BinnenRing := BuitenRing;
        hRouteDistance := hRouteDistance + 1;
      until BuitenRing.CellCount = 0;
    end;{-for}
//  Showmessage(IntToStr(RouteDistance[1,1,4,4]));
end;

procedure THidatoGrid.PlaceNumber(n, i, j: integer);
  var p, n2, i2, j2: integer;
begin
{Vullen Cells en synchroon bijwerken NumberPlacement en CellPossibility
 Deze routine wordt ook gebruikt door ReCalculateNumberPlacementCellPossibility}
  if self.Cells[i,j] = 0 then
  begin
    self.StatCount := self.StatCount + 1;
    self.Proceeded := true;
  end;
  self.Cells[i,j] := n;

{(i,j) verwijderen uit NumberPlacement[n2] voor n2 <> n}
  for p := 1 to CellPossibility[i,j].NumberCount do
  begin
    n2 := CellPossibility[i,j].nsn[p];
    if n2 <> n then
      self.RemoveFromNumberPlacement(n2,i,j);
  end;
  self.CellPossibility[i,j] := NumberSingleton[n];

{n verwijderen uit CellPossibility[i2,j2] voor i2<>i, j2<>j}
  for p := 1 to NumberPlacement[n].CellCount do
  begin
    i2 := NumberPlacement[n].csi[p];
    j2 := NumberPlacement[n].csj[p];
      if not ((i2 = i) and (j2 = j)) then
        self.RemoveFromCellPossibility(n,i2,j2);
  end;
  self.NumberPlacement[n] := CellSingleton[i,j];
end;

procedure THidatoGrid.RemoveFromNumberPlacement(n, i, j: integer);
  var idx: integer;
begin
  if NumberPlacement[n].IsIncluded[i,j] then self.Proceeded := true;

  NumberPlacement[n].IsIncluded[i,j] := false;

  idx := 1;
  while ((NumberPlacement[n].csi[idx] <> i) or (NumberPlacement[n].csj[idx] <> j))
      and ( idx < NumberPlacement[n].CellCount ) do idx := idx + 1;

  if (NumberPlacement[n].csi[idx] = i) and (NumberPlacement[n].csj[idx] = j) then
  begin

    NumberPlacement[n].csi[idx] := 0;
    NumberPlacement[n].csj[idx] := 0;

    while idx < NumberPlacement[n].CellCount do
    begin
      NumberPlacement[n].csi[idx] := NumberPlacement[n].csi[idx + 1];
      NumberPlacement[n].csj[idx] := NumberPlacement[n].csj[idx + 1];
      idx := idx + 1;
    end;{-while}
    NumberPlacement[n].CellCount := NumberPlacement[n].CellCount - 1;
    if NumberPlacement[n].CellCount = 0 then self.Unsolvable := true;
  end;
end;

procedure THidatoGrid.RemoveFromCellPossibility(n, i, j: integer);
  var idx: integer;
begin
  if NumberPlacement[n].IsIncluded[i,j] then self.Proceeded := true;

  idx := 1;
  while (CellPossibility[i,j].nsn[idx] <> n) and ( idx < CellPossibility[i,j].NumberCount ) do idx := idx + 1;

  if CellPossibility[i,j].nsn[idx] = n then
  begin
    CellPossibility[i,j].nsn[idx] := 0;
    while idx < CellPossibility[i,j].NumberCount do
    begin
      CellPossibility[i,j].nsn[idx] := CellPossibility[i,j].nsn[idx + 1];
      idx := idx + 1;
    end;{-while}
    CellPossibility[i,j].NumberCount := CellPossibility[i,j].NumberCount - 1;
    if CellPossibility[i,j].NumberCount = 0 then self.Unsolvable := true;
  end;
end;

procedure THidatoGrid.ExcludeNumberFromCell(n, i, j: integer);
begin
  if self.NumberPlacement[n].IsIncluded[i,j] then
  begin
    self.RemoveFromNumberPlacement(n,i,j);
    self.RemoveFromCellPossibility(n,i,j);
  end;
end;

function THidatoGrid.EqualCellSet(CellSet1, CellSet2: TCellSet): boolean;
  var p: integer;
begin
//BELANGRIJK
//EqualCellSet gaat er impliciet van uit dat de aangeboden cellsets een vaste
// sortering hebben van de arrays csi en csj.
//Dat is het geval zolang de CellSets altijd eerst gevuld worden vanuit constante AllCellSet
// waarna er alleen nog maar cellen uit worden verwijderd
//Binnen AddAllForced en bijbehorende routines is dit het uitgangspunt
//Cellsets met dezelfde elementen maar een andere sortering van csi en csj worden
// hier dus als ongelijk aangemerkt!!
  result := true;
  if CellSet1.CellCount <> CellSet2.CellCount then
  begin
    result := false;
    exit;
  end;
  for p := 1 to CellSet1.CellCount do
  begin
    if (CellSet1.csi[p] <> CellSet2.csi[p]) or (CellSet1.csj[p] <> CellSet2.csj[p]) then
    begin
      result := false;
      exit;
    end;
  end;
end;

function THidatoGrid.EqualNumberSet(NumberSet1, NumberSet2: TNumberSet): boolean;
  var p: integer;
begin
//BELANGRIJK
//EqualNumberSet gaat er impliciet van uit dat de aangeboden Numbersets een vaste
// sortering hebben van de array nsn.
//Dat is het geval zolang de NumberSets altijd eerst gevuld worden vanuit constante AllNumberSet
// waarna er alleen nog maar cijfers uit worden verwijderd
//Binnen AddAllForced en bijbehorende routines is dit het uitgangspunt
//Numbersets met dezelfde elementen maar een andere sortering van nsn worden
// hier dus als ongelijk aangemerkt!!
  result := true;
  if NumberSet1.NumberCount <> NumberSet2.NumberCount then
  begin
    result := false;
    exit;
  end;
  for p := 1 to NumberSet1.NumberCount do
  begin
    if NumberSet1.nsn[p] <> NumberSet2.nsn[p] then
    begin
      result := false;
      exit;
    end;
  end;
end;

procedure THidatoGrid.GetFromStringGrid(var StrGr : TStringGrid);
  var n, i, j : integer;
begin
  self.EmptyCells;
  for i := 1 to 9 do
    for j := 1 to 9 do
    begin
      n := StrToIntDef(StrGr.Cells[j-1,i-1],0);
      self.Cells[i,j] := 0;
      if (n > 0) and (n < 82) then self.Cells[i,j] := n;
    end;{-for}
  PlaceIntoStringGrid(StrGr);
end;

procedure THidatoGrid.PlaceIntoStringGrid(var StrGr : TStringGrid);
  var i, j : Integer;
begin
  for i := 1 to 9 do
    for j := 1 to 9 do
    begin
      if self.Cells[i,j] =  0 then
          StrGr.Cells[j-1,i-1] := '' else
          StrGr.Cells[j-1,i-1] := IntToStr(self.Cells[i,j]);
    end;{-for}
end;

procedure THidatoGrid.PlaceIntoStringList(var StrL : TStringList);
  var i: integer;

  function NumToStr(n : integer) : string;
  begin
    result := IntToStr(n);
    if length(result) = 1 then result := '0' + result;
  end;{-CellToStr}

begin

  StrL.clear;
  for i := 1 to 9 do
  begin
      StrL.Add(NumToStr(self.Cells[i,1])  + ';' +
               NumToStr(self.Cells[i,2])  + ';' +
               NumToStr(self.Cells[i,3])  + ';' +
               NumToStr(self.Cells[i,4])  + ';' +
               NumToStr(self.Cells[i,5])  + ';' +
               NumToStr(self.Cells[i,6])  + ';' +
               NumToStr(self.Cells[i,7])  + ';' +
               NumToStr(self.Cells[i,8])  + ';' +
               NumToStr(self.Cells[i,9]));
  end;{-for}
end;

procedure THidatoGrid.GetFromStringList(var StrL : TStringList);
  var i, j : integer;
begin
  self.EmptyCells;
  for i := 1 to 9 do
    for j := 1 to 9 do
      if length(StrL.Strings[i - 1]) > 3 * j - 2 then
        self.Cells[i,j] := StrToIntDef(copy(StrL.Strings[i - 1],3 * j - 2,2),0);
end;

procedure THidatoGrid.InitNumberPlacement;
  var n: integer;
begin
//NumberPlacement geheel uitwissen, d.h.i. elk cijfer kan overal geplaatst worden
  for n := 1 to 81 do self.NumberPlacement[n] := AllCellSet;
end;

procedure THidatoGrid.InitCellPossibility;
  var i, j: integer;
begin
//CellPossibility geheel uitwissen, d.h.i. elke cel kan elk cijfer bevatten
  for i := 1 to 9 do
    for j := 1 to 9 do
      self.CellPossibility[i,j] := AllNumberSet;
end;

procedure THidatoGrid.DebugNumberPlacement(n: integer);
  var hStr: string;
  var i, j: integer;
  var p: integer;
begin
  hStr := 'NumberPlacement[' + IntToStr(n)  + '] : ';
  hStr := hStr + 'Count: ' + IntToStr(self.NumberPlacement[n].CellCount) + '/ ';
  i := 0;
  j := 0;
  for p := 1 to self.NumberPlacement[n].CellCount do
  begin
    i := self.NumberPlacement[n].csi[p];
    j := self.NumberPlacement[n].csj[p];
    hStr := hStr + '(' + IntToStr(i) + ',' + IntToStr(j) + '), ';
    if not NumberPlacement[n].IsIncluded[i,j] then hStr := hStr + 'NOT INCL!!';
  end;
  if (i = 0) or (j = 0) then hStr := hStr + '...leeg';
  ShowMessage(hStr);
end;

procedure THidatoGrid.DebugCellPossibility(i, j: integer);
  var hStr: string;
  var n: integer;
  var p: integer;
begin
  hStr := 'CellPossibility[' + IntToStr(i) + ',' + IntToStr(j) + '] : ';
  hStr := hStr + 'Count: ' + IntToStr(self.CellPossibility[i,j].NumberCount) + '/ ';
  n := 0;
  for p := 1 to self.CellPossibility[i,j].NumberCount do
  begin
    n := self.CellPossibility[i,j].nsn[p];
    hStr := hStr + IntToStr(n) + ', ';
    if not NumberPlacement[n].IsIncluded[i,j] then hStr := hStr + 'NOT INCL!!';
  end;
  if (n = 0) then hStr := hStr + '...leeg';
  ShowMessage(hStr);
end;

constructor THidatoGame.Create;
begin
  inherited Create;
  self.MainGrid := THidatoGrid.Create;
end;

destructor THidatoGame.Destroy;
begin
  self.MainGrid.Free;
  inherited Destroy;
end;

initialization;
  FillConstants;
end.
