unit Hidatoru;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, FileCtrl, HidLogicu, Grids, ClipBrd, Menus;

type
  THidatorf = class(TForm)
    StringGrid1: TStringGrid;
    MainMenu1: TMainMenu;
    Puzzle: TMenuItem;
    MItmEmptyCells: TMenuItem;
    MItmAddAllForced: TMenuItem;
    EdtValid: TEdit;
    MItmCreateCompleteGrid: TMenuItem;
    EdtMaxCluesWtd: TEdit;
    LMaxCluesWanted: TLabel;
    MItmCreatePuzzle: TMenuItem;
    LPasteCut: TLabel;
    procedure synchronize;
    procedure DisableAll;
    procedure EnableAll;
    procedure GetMaxClWntd;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure StringGrid1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure StringGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure PastePuzzle;
    procedure CutPuzzle;
    procedure MItmEmptyCellsClick(Sender: TObject);
    procedure MItmAddAllForcedClick(Sender: TObject);
    procedure MItmCreateCompleteGridClick(Sender: TObject);
    procedure MItmCreatePuzzleClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private Declarations }
  public
    { Public Declarations }
  end;

var
  Hidatorf: THidatorf;

implementation

{$R *.dfm}

var MyHidatoGame : THidatoGame;
    StrlCijfers  : TStringList;

procedure THidatorf.synchronize;
begin
  MyHidatoGame.MainGrid.GetFromStringGrid(self.StringGrid1);
  if MyHidatoGame.MainGrid.Valid then
      begin
        Hidatorf.EdtValid.Text := 'Valid';
        Hidatorf.EdtValid.Color := clWhite;
        Hidatorf.EdtValid.Font.Style := [];
      end{-else}
    else
      begin
        Hidatorf.EdtValid.Text := 'Not valid';
        Hidatorf.EdtValid.Color := clYellow;
        Hidatorf.EdtValid.Font.Style := [fsBold];
      end;{-if}
  GetMaxClWntd;
end;

procedure THidatorf.DisableAll;
begin
  self.Puzzle.Enabled := false;
  self.StringGrid1.Enabled := false;
  self.EdtMaxCluesWtd.Enabled := false;
  self.LMaxCluesWanted.Font.Color := clGray;
  self.LPasteCut.Font.Color := clGray;
end;

procedure THidatorf.EnableAll;
begin
  self.Puzzle.Enabled := true;
  self.StringGrid1.Enabled := true;
  self.EdtMaxCluesWtd.Enabled := true;
  self.LMaxCluesWanted.Font.Color := clBlack;
  self.LPasteCut.Font.Color := clBlack;
end;

procedure THidatorf.GetMaxClWntd;
begin
  MyHidatoGame.MainGrid.MaxClWntd := StrToIntDef(EdtMaxCluesWtd.Text,0);
  if MyHidatoGame.MainGrid.MaxClWntd < 8 then
  begin
    ShowMessage('Maximum number of clues wanted: 63 substituted.');
    EdtMaxCluesWtd.Text := '63';
    MyHidatoGame.MainGrid.MaxClWntd := 63;
  end;{-if}
end;

procedure THidatorf.FormCreate(Sender: TObject);
begin
  MyHidatoGame := THidatoGame.Create;
end;

procedure THidatorf.FormDestroy(Sender: TObject);
begin
  MyHidatoGame.Free;
end;

procedure THidatorf.StringGrid1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  synchronize;
end;

procedure THidatorf.StringGrid1SelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  synchronize;
end;

procedure THidatorf.PastePuzzle;
begin
  DisableAll;
  Synchronize;
  MyHidatoGame.MainGrid.EmptyCells;
  StrlCijfers := TStringList.Create;
  StrlCijfers.Text := clipboard.AsText;
  MyHidatoGame.MainGrid.GetFromStringList(StrlCijfers);
  StrLCijfers.Free;
  MyHidatoGame.MainGrid.PlaceIntoStringGrid(Hidatorf.StringGrid1);
  Synchronize;
  EnableAll;
end;

procedure THidatorf.CutPuzzle;
begin
  DisableAll;
  Synchronize;
  StrlCijfers := TStringList.Create;
  MyHidatoGame.MainGrid.PlaceIntoStringList(StrlCijfers);
  clipboard.AsText := StrLCijfers.Text;
  StrLCijfers.Free;
  EnableAll;
end;

procedure THidatorf.MItmEmptyCellsClick(Sender: TObject);
begin
  DisableAll;
  synchronize;
  MyHidatoGame.MainGrid.EmptyCells;
  MyHidatoGame.MainGrid.PlaceIntoStringGrid(self.StringGrid1);
  synchronize;
  EnableAll;
end;

procedure THidatorf.MItmAddAllForcedClick(Sender: TObject);
begin
  DisableAll;
  synchronize;
  MyHidatoGame.MainGrid.AddAllForced;
  MyHidatoGame.MainGrid.PlaceIntoStringGrid(Hidatorf.StringGrid1);
  synchronize;
  EnableAll;
end;

procedure THidatorf.MItmCreateCompleteGridClick(Sender: TObject);
begin
  DisableAll;
  synchronize;
  MyHidatoGame.MainGrid.CreateCompleteGrid;
  MyHidatoGame.MainGrid.PlaceIntoStringGrid(Hidatorf.StringGrid1);
  synchronize;
  EnableAll;
end;

procedure THidatorf.MItmCreatePuzzleClick(Sender: TObject);
begin
  DisableAll;
  synchronize;
  MyHidatoGame.MainGrid.CreatePuzzle;
  MyHidatoGame.MainGrid.PlaceIntoStringGrid(Hidatorf.StringGrid1);
  synchronize;
  EnableAll;
end;

procedure THidatorf.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ( Key = vk_F11 ) then PastePuzzle else
               if ( Key = vk_F12 ) then CutPuzzle;
end;

initialization
finalization
end.
